@@ -4,9 +4,10 @@ from numpy.matlib import repmat
 import sys
 import copy
 
-from aislab.gnrl.bf import *  # c_
+from aislab.gnrl.bf import* # c_
 from aislab.gnrl.sf import *
-from aislab.gnrl.measr import *  # vaf
+from aislab.gnrl.tm import *
+from aislab.gnrl.measr import* # vaf
 
 
 ###################################################################################
@@ -130,7 +131,7 @@ def bns2arr(bns):
     for i in bns[1]['y']:
         luy = len(bns[1]['y'][i])
         # j = np.arange(j[len(j)-1] + 1 ,j[len(j)-1]+1+luy)
-        wy = np.append(wy, repmat(1 / luy, 1, luy))
+        wy = np.append(wy, repmat(1/luy, 1, luy))
     return n, nw, syy, syy2, wy
 
 
@@ -170,15 +171,15 @@ def binst(bins, mod, y, w, ytp, yi):
                     bins['woe'][i + 1] = []
             else:
                 if len(y):
-                    b[i + 1] = [np.sum(w * yh)]
-                    b2[i + 1] = [np.sum(w * yh ** 2)]
+                    b[i + 1] = [np.sum(w*yh)]
+                    b2[i + 1] = [np.sum(w*yh ** 2)]
                 else:
                     b[i + 1] = [0]
                     b2[i + 1] = [0]
                 bins['y'] = b
                 bins['y2'] = b2
                 if mod == 'stats':
-                    bins['my'][i + 1] = b[i + 1][0] / max(bins['nw'], 1e-12)
+                    bins['my'][i + 1] = b[i + 1][0]/max(bins['nw'], 1e-12)
                     bins['woe'][i + 1] = []
     return bins
 
@@ -201,20 +202,20 @@ def cut(x, cuts):
 ###################################################################################
 def cnd_gbi(gg, ngg, dGBInd, tolmindGBInd, Dind):
     GBI = gbi(gg, ngg)
-    GBI1 = GBI + 200 * (GBI < 0)
+    GBI1 = GBI + 200*(GBI < 0)
     dGBI = diff(GBI1)
     # sdGBI = np.sort(dGBI)
-    cnd_GBI = np.min(abs(dGBI), axis=0) >= dGBInd - tolmindGBInd * Dind
+    cnd_GBI = np.min(abs(dGBI), axis=0) >= dGBInd - tolmindGBInd*Dind
     return cnd_GBI
 
 
 ###################################################################################
 def cnd_trnd(ns, gg, nn, md, mt):
     if mt is not np.nan:
-        my = gg / nn
+        my = gg/nn
         my[my < 1e-3] = 1e-3
         dmy = diff(my)
-        trnd = (np.sum(dmy > 0, axis=0) == md - 1) * 1 - (np.sum(dmy < 0, axis=0) == md - 1) * 1
+        trnd = (np.sum(dmy > 0, axis=0) == md - 1)*1 - (np.sum(dmy < 0, axis=0) == md - 1)*1
         if mt == 0:
             cnd_TRND = abs(trnd) == 1
         else:
@@ -229,7 +230,7 @@ def cnd_trnd(ns, gg, nn, md, mt):
 #   encoding
 #   - met = woe, mdep, ...
 ###################################################################################
-def enc_int(x, cname, xtp, vtp, order, dsp=0, dlm=' '):
+def enc_int(x, cname, xtp, vtp, order, dsp=False, dlm=' '):
     N = x.shape[0]
     xnum = np.full((len(x), len(cname)), np.nan)
     for i in range(len(cname)):
@@ -372,30 +373,30 @@ def inv_distr(DistrType, pval, df1, df2, x1, x2, tol):
             fa = fb
             fb = fc
             fc = fa
-        tol1 = 2 * EPS * abs(b) + 0.5 * tol
-        xm = 0.5 * (c - b)
+        tol1 = 2*EPS*abs(b) + 0.5*tol
+        xm = 0.5*(c - b)
         cnd1 = abs(b) < 1e-10
         if abs(xm) <= tol1 or fb == 0:
-            return int(cnd) * (cnd1 * 1e10 + int(not cnd1) * (1 / b - 1) * df1 / df2) + int(not cnd) * 2 * b
+            return int(cnd)*(cnd1*1e10 + int(not cnd1)*(1/b - 1)*df1/df2) + int(not cnd)*2*b
         if abs(e) >= tol1 and abs(fa) > abs(fb):
-            s = fb / fa
+            s = fb/fa
             if a == c:
-                p = 2 * xm * s
+                p = 2*xm*s
                 q = 1 - s
             else:
-                q = fa / fc
-                r = fb / fc
-                p = s * (2 * xm * q * (q - r) - (b - a) * (r - 1))
-                q = (q - 1) * (r - 1) * (s - 1)
+                q = fa/fc
+                r = fb/fc
+                p = s*(2*xm*q*(q - r) - (b - a)*(r - 1))
+                q = (q - 1)*(r - 1)*(s - 1)
             if p > 0:
                 q = -q
             p = abs(p)
-            min1 = 3 * xm * q - abs(tol1 * q)
-            min2 = abs(e * q)
+            min1 = 3*xm*q - abs(tol1*q)
+            min2 = abs(e*q)
             cnd2 = min1 < min2
-            if 2 * p < (cnd2 * min1 + int(not cnd2) * min2):
+            if 2*p < (cnd2*min1 + int(not cnd2)*min2):
                 e = d
-                d = p / q
+                d = p/q
             else:
                 d = xm
                 e = d
@@ -407,18 +408,18 @@ def inv_distr(DistrType, pval, df1, df2, x1, x2, tol):
         if abs(d) > tol1:
             b = b + d
         else:
-            prod = xm * tol1
+            prod = xm*tol1
             if prod == 0:
                 sgn = 1
             else:
                 sgn = np.sign(prod)
-            b = b + sgn * abs(tol1)
+            b = b + sgn*abs(tol1)
         if cnd:
             fb = betai(df1, df2, b)[0] - pval
         else:
             fb = gamq(df2, b)[0] - pval
     cnd1 = abs(b) < 1e-10
-    return cnd * (cnd1 * 1e10 + int(not cnd1) * (1 / b - 1) * df1 / df2) + int(not cnd) * 2 * b
+    return cnd*(cnd1*1e10 + int(not cnd1)*(1/b - 1)*df1/df2) + int(not cnd)*2*b
 
 
 ###################################################################################
@@ -428,13 +429,13 @@ def mdep(syy, nw, wy, w=[]):
         m = 1; [Luy] = syy.shape
     else:
         [m, Luy] = syy.shape
-    my = syy / repmat(nw, 1, Luy)
+    my = syy/repmat(nw, 1, Luy)
     mmy = my.max(axis=0)
     if not isinstance(mmy, np.ndarray): mmy = np.array([[mmy]])
     mmy[mmy < 1e-6] = 1e-6  # !!! works for ytp = 'bin'
-    if len(w) == 0: w = r_(wy / mmy)
+    if len(w) == 0: w = r_(wy/mmy)
     ww = repmat(w, m, 1)
-    x = my * ww
+    x = my*ww
     if not isinstance(w, np.ndarray): w = np.array([[w]])
     return x, w
 
@@ -514,15 +515,15 @@ def nabub(x, w, sv, xtp, ux, y, ytp, yi):
 ###################################################################################
 def passChi2(df, df_den, bm12, pt, ytp, ba):
     tol = 1e-3
-    df2 = df / 2
-    df_den2 = df_den / 2
+    df2 = df/2
+    df_den2 = df_den/2
     if ytp[0] == 'bin':
         if ba:
-            apt = pt / bm12
+            apt = pt/bm12
         else:
             apt = pt
         left = 0
-        right = -4 * df2 * np.log(apt)
+        right = -4*df2*np.log(apt)
         dummy = 1;
         if apt < 1e-200:
             chi_pass = right
@@ -558,13 +559,13 @@ def permints(n, m, init=True):
 
 ###################################################################################
 def reduce_bns1(bns, xtp, g, b, nw, gd, bd):
-    order_nom = lambda g, nw: g / max(1, nw)
-    cnd_mrg = lambda g, b, nw, gd, bd, p1: g < gd and g * p1 < 1 or b < bd and b * p1 < 1  # merge condition
+    order_nom = lambda g, nw: g/max(1, nw)
+    cnd_mrg = lambda g, b, nw, gd, bd, p1: g < gd and g*p1 < 1 or b < bd and b*p1 < 1  # merge condition
     bng = copy.deepcopy(bns)
     if xtp == 'nom':
         bns = sort_bins(config, bns, order_nom, BUS_groups)
     try:
-        pg = b.sum() / nw.sum()
+        pg = b.sum()/nw.sum()
         p1 = min([pg, 1 - pg])
     except:
         print('Error: Binning stats are wrong...')  # todo: make it real error
@@ -592,7 +593,7 @@ def reduce_bns1(bns, xtp, g, b, nw, gd, bd):
 def reduce_bns2(bns, ytp, md, mar):
     lpm = np.log(1e4)
     kk = np.arange(md + 2, 1e5 + 1)
-    kkk = ((kk - 1) * np.log(kk - 1) - (kk - md) * np.log(kk - md) - (md - 1) * np.log(md - 1))[::-1]
+    kkk = ((kk - 1)*np.log(kk - 1) - (kk - md)*np.log(kk - md) - (md - 1)*np.log(md - 1))[::-1]
     ind = np.arange(len(kkk))[::-1]
     mmax = ind[kkk < lpm].max() + md + 2 + 1
 
@@ -610,7 +611,7 @@ def reduce_bns2(bns, ytp, md, mar):
             if any(cnd):
                 cnt_max = cnd.index(True)
             else:
-                dC = (y1 + y2) ** 2 / (n1 + n2) - y1 ** 2 / n1 - y2 ** 2 / n2
+                dC = (y1 + y2) ** 2/(n1 + n2) - y1 ** 2/n1 - y2 ** 2/n2
                 dC[dC > 0] = 0
                 cnd = n1 + n2 <= mar
                 isc = np.arange(1, len(cnd) + 1)[cnd.flatten()]
@@ -647,15 +648,17 @@ def reduce_bns2(bns, ytp, md, mar):
 ###################################################################################
 def sbng(bng, md=5, skip_mos=['m', 'o', 'sv'], met_dist='manh', met='co1y', nmin_uy=[50, 50], mar=np.inf, pt=0.05, ba=0,
          maxprm=20000, dGBInd=5, tolmindGBInd=5, mt=np.nan, minB=50, tolminB=20, stats=True, dsp=False):
-    # bng1 = copy.deepcopy(bng)
-    # del bng
-    # bng = bng1
+    bng1 = copy.deepcopy(bng)
+    del bng
+    bng = bng1
+    del bng1
     if isinstance(bng, list):
         m = len(bng)
     else:
         m = 1
-    sb = [None] * m
+    sb = [None]*m
     for i in range(m):
+        if dsp: tic5()
         sb[i] = {}
         if m == 1:
             bns = bng['bns'];       cname = bng['cname'];       xtp = bng['xtp'];       ytp = bng['ytp']
@@ -697,7 +700,7 @@ def sbng(bng, md=5, skip_mos=['m', 'o', 'sv'], met_dist='manh', met='co1y', nmin
         sb[i]['st']['D'] = D
         if stats: st, sb[i]['bns'] = stbng('sb', sb[i]['bns'], xtp, ytp)
 
-        if dsp: print('Run SB: ', i + 1, '/', m, cname, ',      time: ', toc(0))
+        if dsp: print('Run SB: ', i + 1, '/', m, cname, ',      time: ', toc5(dsp=False))
 
     if m == 1:
         return sb[0]
@@ -767,7 +770,7 @@ def setbin(x, w, xtp, btp, y=np.empty((0, 0)), ytp='', yi=[]):
 def setyi(y, ytp):
     if isinstance(ytp, str): ytp = [ytp]
     r = len(ytp)
-    yi = [None] * r
+    yi = [None]*r
     for i in range(r):
         if ytp[i] == 'num' or ytp[i] == 'bin':
             yi[i] = []
@@ -834,10 +837,9 @@ def sort_bins(vname, config, unordered_bins, bin_order_function):
 ###################################################################################
 def spih(bns, xtp, md, nd, met):
     Nw = 0
-    for k in bns:
-        Nw = Nw + bns[k]['nw']
+    for k in bns: Nw = Nw + bns[k]['nw']
     m = len(bns)
-    Nd = Nw / md
+    Nd = Nw/md
     i = 0
     iold = 1
     s = 0
@@ -915,7 +917,7 @@ def spih_erv(x, w, xtp, md, sv):
     x = x[~ir[0]]
     xmin = min(x)
     xmax = max(x)
-    s = (xmax - xmin) / md
+    s = (xmax - xmin)/md
     b = np.arange(xmin, xmax + s, s)
     b = np.round(b, 4)
     lb = b[0:len(b) - 1]
@@ -951,7 +953,7 @@ def spih_sb_co1y(bns, xtp, ytp, md, gd, bd, mar, pt, ba, dGBInd, tolmindGBInd, m
         pass  # Not implemented for non-binary dependant
     mm = range(2, min([len(bns), md]) + 1)  # number of bins to try
     crit_b = nans((md - 1,))
-    bns_b = [None] * (md - 1)
+    bns_b = [None]*(md - 1)
     pvalc_b = nans((md - 1,))
     br = 0
     for md in mm:
@@ -965,7 +967,7 @@ def spih_sb_co1y(bns, xtp, ytp, md, gd, bd, mar, pt, ba, dGBInd, tolmindGBInd, m
         Nw = sum(nw)
         df = md - 1
         df_den = Nw - md
-        Chi2pass = passChi2(df, df_den, bm1 * bm2, pt, ytp, ba)
+        Chi2pass = passChi2(df, df_den, bm1*bm2, pt, ytp, ba)
         s2 = permints(m, md)  # s2 - set of all merges to be investigated
         if s2.shape[0] != 0:
             gg, nn = stbngs(s2, g, nw)
@@ -983,7 +985,7 @@ def spih_sb_co1y(bns, xtp, ytp, md, gd, bd, mar, pt, ba, dGBInd, tolmindGBInd, m
 
             cnd_TRND = cnd_trnd(s2.shape[0], gg, nn, md, mt)
             cnd_pt = Chi2 >= Chi2pass
-            cnd_minB = np.min(ngg, axis=0) >= minB - tolminB * Dind
+            cnd_minB = np.min(ngg, axis=0) >= minB - tolminB*Dind
             cnd_mar = np.max(nn, axis=0) <= mar
             cnd = cnd_GBI & cnd_TRND & cnd_pt & cnd_minB & cnd_mar
             if cnd.any():
@@ -1104,22 +1106,22 @@ def stbng(met, bns, xtp, ytp, x=np.empty((0, 0)), y=np.empty((0, 0)), w=np.empty
             s['sy'] = sy
             syb = np.array(nw - sy)
             s['syb'] = syb
-            s['pnw'] = nw / Nw * 100
-            s['py0'] = np.dot(sy, np.linalg.pinv(Sy[np.newaxis])) * 100
-            s['py1'] = np.dot(syb, np.linalg.pinv([Nw - Sy])) * 100
+            s['pnw'] = nw/Nw*100
+            s['py0'] = np.dot(sy, np.linalg.pinv(Sy[np.newaxis]))*100
+            s['py1'] = np.dot(syb, np.linalg.pinv([Nw - Sy]))*100
             temp = s['py0']; temp[np.isnan(temp)] = 0
             s['cp0'] = np.cumsum(temp)
             temp = s['py0']; temp[np.isnan(temp)] = 0
             s['cp1'] = np.cumsum(temp)
             nw1 = nw
             nw1[nw < 1e-12] = 1e-12
-            s['my0'] = np.array(syb / nw1)
-            s['my1'] = np.array(sy / nw1)
+            s['my0'] = np.array(syb/nw1)
+            s['my1'] = np.array(sy/nw1)
             s['gbind'] = gbi(sy, syb)
             syb1 = syb
             syb1[syb < 1e-12] = 1e-12
 
-            s['odds'] = sy / syb
+            s['odds'] = sy/syb
             s['Gini'] = gini(sy, syb)
             s['WoE'] = woe(sy, syb)
             s['KS'] = ks(sy, syb)
@@ -1129,7 +1131,7 @@ def stbng(met, bns, xtp, ytp, x=np.empty((0, 0)), y=np.empty((0, 0)), w=np.empty
             s['Chi2'] = chi2(sy, syb)
         elif ytpi == 'num':
             s['sy'] = sy
-            s['pnw'] = nw / Nw * 100
+            s['pnw'] = nw/Nw*100
             s['Frat'] = frat(sy, sy2, nw)
         else:
             s = []
@@ -1170,27 +1172,23 @@ def ubng(x, xtp, w, md=100, prcd=1, nmin=50, met='enr', sv=[], xi_order=[], y=np
          cnames=None, stats=False, dsp=False):
     m = x.shape[1]
     if cnames is None: cnames = ['var_' + str(i) for i in range(m)]
-    ub = [None] * m
-    if isinstance(xtp, str): xtp = [xtp] * m
+    ub = [None]*m
+    if isinstance(xtp, str): xtp = [xtp]*m
     # if isinstance(sv, str): xtps = [sv]*m
     # if isinstance(xi_order, str): xtps = [xi_order]*m
     # if isinstance(mt_order, str): xtps = [mt_order]*m
     for i in range(m):
-        tic()
+        if dsp: tic5()
         ub[i] = {}
-
-        if isinstance(x, pd.DataFrame):
-            xi = c_(x.iloc[:, i].values)
-        else:
-            xi = x
-
+        if isinstance(x, pd.DataFrame): xi = c_(x.iloc[:, i].values)
+        else:                           xi = x
         xtpi = xtp[i]
         if len(y) == 0:
             ytp = ''
             yi = []
         else:
             if len(yi) == 0: yi = setyi(y, ytp)
-        if met == 'epp': md = 100 / prcd
+        if met == 'epp': md = 100/prcd
         N = len(xi)
         if met == 'enr' or met == 'epp' or met == 'svc':
             bns = nabub(xi, w, sv, xtpi, xi_order, y, ytp, yi)
@@ -1218,7 +1216,7 @@ def ubng(x, xtp, w, md=100, prcd=1, nmin=50, met='enr', sv=[], xi_order=[], y=np
         ub[i]['cname'] = cnames[i]
         ub[i]['xtp'] = xtpi
         ub[i]['ytp'] = ytp
-        print('Run UB: ', i + 1, '/', m, cnames[i], ',      time: ', toc(0))
+        if dsp: print('Run UB: ', i + 1, '/', m, cnames[i], ',      time: ', toc5(dsp=False))
 
     if m == 1:
         return ub[0]
