@@ -401,7 +401,7 @@ def pvalF(F, v1, v2, tp='ot'):
     p[p > 1] = 1  #
     return p
 ###################################################################################
-def vaf(Y, Ym, w=np.empty([0, 0]), p=0):
+def vaf(Y, Ym, w=np.empty([0, 0]), p=0, floor_to_0=True):
     Nw, r = Y.shape
     if w.size > 0:
         if w.size < Y.size: w = np.tile(w, (1, r))
@@ -414,7 +414,7 @@ def vaf(Y, Ym, w=np.empty([0, 0]), p=0):
         SSE = sum((Y - Ym)**2)
         SST = sum((Y - mY)**2)
     VAF = 1 - (SSE/ SST*(Nw - 1)/(Nw - p - 1))
-    VAF[VAF < 0] = 0
+    if floor_to_0: VAF[VAF < 0] = 0
     return VAF * 100
 ###################################################################################
 def vif(x):
