@@ -379,7 +379,7 @@ def stmdl(FB=None, model=None, FY=None, P=None, wF=None, n2=np.nan, n1=np.nan, s
 
 
 ###################################################################################
-def swlinr(x=None, y=None, w=None, cnames=None, pm0=1, met='SWR', SLE=0.05, SLS=0.05, crit_nbm=['Cp', 'AIC'], mtp='empty', ivi=np.empty([1,]).astype(int), val_prc=0, s_min=1e-12, dsp='all'):
+def swlinr(x=None, y=None, w=None, cnames=None, pm0=1, met='SWR', SLE=0.05, SLS=0.05, crit_nbm=['Cp', 'AIC'], mtp='empty', ivi=np.empty([1,]).astype(int), val_prc=0, s_min=1e-12, dsp=False):
     N = x.shape[0]
     if cnames is None: cnames = ['var_' + str(i) for i in range(m)]
     if pm0 == 1:
@@ -398,7 +398,7 @@ def swlinr(x=None, y=None, w=None, cnames=None, pm0=1, met='SWR', SLE=0.05, SLS=
 
 ###################################################################################
 def visualz(model, iterate, dsp, mtp):
-    if dsp == 'no': return
+    if not dsp: return
     model0 = copy.deepcopy(model)
     i2 = iterate
     if i2 == -1:
