@@ -315,7 +315,7 @@ def scrChiSq(x, y, ym0, ym, w, st0, model, mode, ivi, ivo, smin):
 
 
 ###################################################################################
-def swlogr(x, y, w, cnames=None, SLE=0.05, SLS=0.05, pm0=1, dsp='all', nbm_crit=np.array([]), mtp='full', met='SWR',
+def swlogr(x, y, w, cnames=None, SLE=0.05, SLS=0.05, pm0=1, dsp=False, nbm_crit=np.array([]), mtp='full', met='SWR',
            metpminit='zero', ivi=np.empty([1, ]).astype(int), met_opt='newt', gcnv=1e-8, fcnv=0, afcnv=0, xcnv=0,
            maxiter=100, smin=1e-12, dsp_op=np.array([1, 2]), smeth='fprev2', p_incr=1.2, p_decr=0.6, sl=1e-12, su=1,
            sc=None):
@@ -477,10 +477,9 @@ def stmdl(FB, x, y, ym, w, st0, model, n2, ivi, ivo, smin):
 
 ###################################################################################
 def visualz(model, iterate, dsp):
-    if dsp == 'no': return
+    if not dsp: return
     step = len(model) - iterate
-    print('Step: ', step,
-          '  =================================================================================================================')
+    print('Step: ', step, '  =================================================================================================================')
     modelInt = model[0]
     model = model[-1]
     FB = model['FB']
@@ -651,4 +650,4 @@ def visualz(model, iterate, dsp):
         print('------------------------------------------------------')
         print(' ')
 
-    if dsp == 'all': toc2()
+    # if dsp == 'all': toc2()
