@@ -9,8 +9,8 @@ from numpy import matlib
 
 from aislab.gnrl.bf import * # c_
 from aislab.gnrl.sf import *
-from aislab.gnrl.measr import *  # vaf
-from aislab.op_nlopt import stopcrt
+from aislab.gnrl.measr import * # vaf
+from aislab.op_nlopt import * # stopcrt
 
 ##############################################################################
 # def dm2dv():
