@@ -3,7 +3,7 @@ import pandas as pd
 
 from aislab.gnrl.sf import *
 from aislab.gnrl.measr import *
-from aislab.op_nlopt.cstm import *
+from aislab.tests.cstm_ga_scoring import *
 
 ##############################################################################
 
